/* ***********************************
 * dirs.c provides a directory traversal
 * library for pimpFM.  More than anything 
 * else it's alot of string manipulation 
 * in c.
 * **********************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dir.h>
#include <unistd.h>
#include "dirs.h"

enum view currentView;  //a lot of stuff depends on this

void moveToNext() {
	if (g_list_next(dirHistory) != NULL) {
		dirHistory = g_list_next(dirHistory);
		current = dirHistory->data;
		killPFiles();
		makePFiles(currentView);

	}
}

void moveToPrev() {
	if (g_list_previous(dirHistory) != NULL) {
		dirHistory = g_list_previous(dirHistory);
		current = dirHistory->data;
		killPFiles();
		makePFiles(currentView);
	}
}

void moveUp () {
	gint i, length;
	gchar *temp;
	length = 0;
	for (i = 0; current[i] != '\0'; i++) {
		if (current[i] == '/' && current[i + 1] != '\0') {
			length = i;
		}
	}
	temp = g_new (gchar, length + 2);
	for (i = 0; i <= length; i++) {
		temp[i] = current[i];
	}
	temp[i] = '\0';
	dirHistory = g_list_append(dirHistory, temp);
	dirHistory = g_list_last(dirHistory);
	current = dirHistory->data;
	killPFiles();
	makePFiles( currentView);
}

void moveToHome() {
	moveToDir (homedir);
}

void moveToDir(char * dir) {
	if (dir[0] == '/') {//see if path is absolute g_path_is_absolute(char) would also work
		dirHistory = g_list_append(dirHistory, dir);
		dirHistory = g_list_last(dirHistory);
		current = dirHistory->data;
		killPFiles();
		makePFiles(currentView);
	} else {
		dirHistory = g_list_append(dirHistory, g_strconcat(current, dir));
		dirHistory = g_list_last(dirHistory);
		current = dirHistory->data;
		killPFiles();
		makePFiles(currentView);
	}
}

void killPFiles() {
	if (pFiles != NULL) {
		while (g_list_next(pFiles) != NULL) {
			g_free(((pFile *) pFiles->data)->name);
			g_free(((pFile *) pFiles->data)->nameTotal);
			g_free(pFiles->data);
			pFiles = g_list_next(pFiles);
		}
		g_list_free(pFiles);
	}
	pFiles = NULL;
}

void makePFiles(int dim) {
	int count, i;
	int *x, *y, *z;
	struct direct **content;
	x = (int *) malloc (sizeof (int));
	y = (int *) malloc (sizeof (int));
	z = (int *) malloc (sizeof (int));
	*x = 0;
	*y = 0;
	*z = 0;
	count = scandir(current, &content, makeHelper, NULL);
	for (i = 0; i < count; i++) {
		nextPos(x, y, z, dim);
		pFiles = g_list_append (pFiles, makePFile(content[i]->d_name, *x, *y, *z));
	}
	pFiles = g_list_first(pFiles);
	((pFile *) pFiles->data)->selected = TRUE;
	selected = (pFile *) pFiles->data;
	free (x);
	free (y);
	free (z);
}

void nextPos (int * x, int* y, int* z, enum view dim) {
	//this is just code to show something
	//I'll make a better algorthm later
	if (*x == 0) {
		*x = 7;
	}
	if (dim != VIEW2D) {
		*x += 8;
	} else {
		*x += 13;
	}
	if (*x > 40) {
		*x = 7;
		*y += 15; //should be changed to a -=
	}
	if (dim == VIEW2D) {
		*z = 0;
	} else {
		if (*y > 40) {
			*z += 15;
			*y = 6;
		}
	}
}

int makeHelper (struct direct * dir) {
	if (displayMode == HIDE) {
		if (dir->d_name[0] == '.') {
			//g_print ("I ran\n");
			return 0;
		} else {
			return 1;
		}
	} else {
		if ((strcmp(dir->d_name, ".") == 0) || (strcmp(dir->d_name, "..") == 0)) {
			return 0;//for false (to shut up compiler
		} else {
			return 1;//for true
		}
	}
}
		
pFile * makePFile( char * file, int x, int y, int z) {
	pFile * temp;
	char * tempC;
	temp = (pFile *) malloc (sizeof (pFile));
	temp->name = malloc((sizeof (char)) * ((strlen(file)) + 1));
	temp->name = strcpy(temp->name, file);
	temp->nameTotal = malloc ((sizeof (char)) * (strlen(file) + (strlen(current) + 2)));
	temp->nameTotal = strcpy (temp->nameTotal, current);
	tempC = temp->nameTotal;
	tempC += strlen(current);
	tempC[0] = '/';
	tempC++;
	//&((temp->nameTotal)[strlen(current)]);
	strcpy (tempC, file);
	temp->filetype = grabFT(file);
	temp->x = x;
	temp->y = y;
	temp->z = z;
	temp->AllGood = stat (temp->nameTotal, &(temp->data));
	(temp->AllGood)--;
	temp->selected = 0;
	//g_print ("%s" , temp->nameTotal);
	temp->angle = 0.0;
	temp->copied = FALSE;
	return temp;
}
enum mime grabFT(char * file) {
	if (strstr(file, ".pl") != NULL) {
		return PL;
	} else if (strstr(file, ".txt") != NULL) {
		return TXT;
	} else if (strstr(file, ".doc") != NULL) {
		return DOC;
	} else if (strstr (file, ".cpp") != NULL) {
		return CPP;  //cpp is listed before c because E c cpp
	} else if (strstr (file, ".el") != NULL) {
		return EL;
	} else if (strstr (file, ".zip") != NULL) {
		return ZIP;
	} else if (strstr (file, ".jpg") != NULL) {
		return JPG;
	} else return BLAH;
}
void repositionFiles(enum view dim) {
	int *x, *y, *z;
	x = malloc (sizeof (int));
	y = malloc (sizeof (int));
	z = malloc (sizeof (int));
	*x = 0;
	*y = 0;
	*z = 0;
	pFiles = g_list_first(pFiles);
	while (g_list_next(pFiles) != NULL) {
		nextPos(x, y, z, dim);
		((pFile *) (pFiles->data))->x = *x;
		((pFile *) (pFiles->data))->y = *y;
		((pFile *) (pFiles->data))->z = *z;
		pFiles = g_list_next(pFiles);
	}
	nextPos(x, y, z, dim);
	((pFile *) (pFiles->data))->x = *x;
	((pFile *) (pFiles->data))->y = *y;
	((pFile *) (pFiles->data))->z = *z;
	free(x);
	free (y);
	free (z);
}
